/*
  Macro           EditRemote
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows TSE v4.2 upwards
  Version         0.1

  One of my possible future projects is to make a TSE extension that lets TSE
  open remote files over SSH.

  It would be an OK user experience to create this as a tool that has its own
  prompt.

  But it would be a great user experience if it could be created as an
  extension that intercepts the initial name that is entered in TSE's own
  Edit(This)File prompt.

  My thought is, that this could be implemented by a _BEFORE_FILE_OPEN_ hook,
  that executes before these commands do anything with the entered file or
  directory name. So in case of a directory name, even before browsing it
  starts.

  An extension could check the remote name for an external protocol like
  "ssh://" and "https://", and change the name to a dummy file for TSE to load.
  After TSE is done loading the dummy file, the extension could replace it with
  the remote file and rename it to its original name.
  For directories it could browse the remote directory.

  This macro is a simplified example of how such an extension could work.
  It only implements loading the source code of a website.
  Based on the compiler directive "FAKE_IT" it either shows (and currently
  fails to compile) the use of the not-yet-existing _BEFORE_FILE_OPEN_ hook,
  or it tests the rest of the code by opening https://semware.com.
*/


/*
  If FAKE_IT is FALSE, then the existence of the new _BEFORE_FILE_OPEN_ hook is
  assumed.
  If FAKE_IT is TRUE, then the code of Semware's web page is downloaded using
  and testing the rest of the code.
*/
#define FAKE_IT TRUE


// Global constants and semi-constants
#define CCF_FLAGS                   _DONT_PROMPT_|_DONT_EXPAND_|_OVERWRITE_
string  MACRO_NAME [MAXSTRINGLEN] = ''


// Global variables
string protocol    [MAXSTRINGLEN] = ''
string remote_name [MAXSTRINGLEN] = ''


dll "<urlmon.dll>"
  integer proc URLDownloadToFile(
    integer pCaller,
    string  szURL     : CSTRVAL,
    string  szFileName: CSTRVAL,
    integer dwReserved,
    integer lpfnCB): "URLDownloadToFileA"
end


proc download_url_to_file(string url, string file)
  EraseDiskFile(file)
  URLDownloadToFile(0, url, file, 0, 0)
end download_url_to_file


string proc create_random_name(integer len)
  integer ascii_code          = 0
  integer i                   = 0
  string  name [MAXSTRINGLEN] = ''
  for i = 1 to len
    ascii_code = Random(48, 109)
    if ascii_code > 57
      ascii_code = ascii_code + 7   // Skip characters between "9" and "A".
    endif
    if ascii_code > 90
      ascii_code = ascii_code + 6   // Skip characters between "Z" and "a".
    endif
    name = name + Chr(ascii_code)
  endfor
  return(name)
end create_random_name


proc before_file_open()
  string local_name [MAXSTRINGLEN] = ''

  if Pos('http://' , Lower(CurrFilename()))
  or Pos('https://', Lower(CurrFilename()))
    protocol    = 'http'
    remote_name = CurrFilename()
    local_name  = create_random_name(MAXSTRINGLEN)
    ChangeCurrFilename(local_name, CCF_FLAGS)
  else
    protocol    = ''
    remote_name = ''
  endif
end before_file_open


proc on_first_edit()
  string tmp_file [MAXSTRINGLEN] = ''
  case protocol
    when 'http'
      tmp_file = GetEnvStr('TMP') + '\TSE_' + MACRO_NAME + '.tmp'
      download_url_to_file(remote_name, tmp_file)
      LoadBuffer(tmp_file)
      EraseDiskFile(tmp_file)
      ChangeCurrFilename(remote_name, CCF_FLAGS)
  endcase
end on_first_edit


proc WhenLoaded()
  MACRO_NAME = SplitPath(CurrMacroFilename(), _NAME_)

  #if not FAKE_IT
    Hook(_BEFORE_FILE_OPEN_, before_file_open)
    Hook(_ON_FIRST_EDIT_   , on_first_edit)
  #endif
end WhenLoaded


proc Main()
  #if FAKE_IT
    NewFile()
    ChangeCurrFilename('https://semware.com', CCF_FLAGS)
    before_file_open()
    on_first_edit()
  #endif
end Main

